# Generated by roxygen2: do not edit by hand

S3method(in_linear_sequence,Date)
S3method(in_linear_sequence,POSIXct)
S3method(in_linear_sequence,character)
S3method(in_linear_sequence,numeric)
S3method(in_range,character)
S3method(in_range,default)
S3method(is_linear_sequence,Date)
S3method(is_linear_sequence,POSIXct)
S3method(is_linear_sequence,character)
S3method(is_linear_sequence,numeric)
S3method(lacking,data.frame)
S3method(lacking,default)
S3method(print,validations)
S3method(satisfying,data.frame)
S3method(satisfying,default)
S3method(summary,validations)
S3method(violating,data.frame)
S3method(violating,default)
export("%vin%")
export("created<-")
export("description<-")
export("label<-")
export("meta<-")
export("origin<-")
export(.PKGOPT)
export(.blocks_expressionset)
export(.get_exprs)
export(.ini_expressionset_cli)
export(.ini_expressionset_df)
export(.ini_expressionset_yml)
export(.show_expressionset)
export(add_indicators)
export(all_complete)
export(all_unique)
export(as.data.frame)
export(as_yaml)
export(cells)
export(check_that)
export(compare)
export(confront)
export(contains_at_least)
export(contains_at_most)
export(contains_exactly)
export(created)
export(description)
export(do_by)
export(does_not_contain)
export(errors)
export(estat_codelist)
export(event)
export(exists_any)
export(exists_one)
export(export_yaml)
export(expr)
export(field_format)
export(field_length)
export(glob)
export(global_codelist)
export(hb)
export(hierarchy)
export(in_linear_sequence)
export(in_range)
export(indicator)
export(is_complete)
export(is_linear_sequence)
export(is_unique)
export(keyset)
export(label)
export(lacking)
export(lbj_cells)
export(lbj_rules)
export(match_cells)
export(max_by)
export(mean_by)
export(meta)
export(min_by)
export(number_format)
export(origin)
export(part_whole_relation)
export(reset)
export(run_validation_dir)
export(run_validation_file)
export(rx)
export(satisfying)
export(sdmx_codelist)
export(sdmx_endpoint)
export(sum_by)
export(validate_options)
export(validator)
export(validator_from_dsd)
export(values)
export(variables)
export(violating)
export(voptions)
exportClasses(lbj_cells)
exportClasses(lbj_rules)
exportMethods("+")
exportMethods("[")
exportMethods("[<-")
exportMethods("[[<-")
exportMethods("created<-")
exportMethods("description<-")
exportMethods("event<-")
exportMethods("label<-")
exportMethods("meta<-")
exportMethods("names<-")
exportMethods("origin<-")
exportMethods(aggregate)
exportMethods(all)
exportMethods(any)
exportMethods(as.data.frame)
exportMethods(barplot)
exportMethods(compare)
exportMethods(event)
exportMethods(keyset)
exportMethods(length)
exportMethods(names)
exportMethods(plot)
exportMethods(sort)
exportMethods(summary)
exportMethods(warnings)
import(grid)
import(methods)
import(settings)
import(yaml)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(stats,complete.cases)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
useDynLib(validate, .registration=TRUE)
